/* -*- Mode: Java; tab-width: 2; c-basic-offset: 2; -*-
 * 
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 *
 * Contributor(s):
 *  Doug Daniels <rainking@rice.edu> -- morphed code for Helper Applications pref window
 *                                      into multiple servers pref window for Annozilla
 *                   
 */


// Constants

const PROFILE_DIRECTORY = "ProfD";
const SERVERS_FILENAME = "annozillaServers.rdf";
const SERVERS_ROOT_URN = "urn:annotations:servers";


const NS_LOCAL_FILE_CONTRACTID = "@mozilla.org/file/local;1";
const NS_LOCALFILEOUTPUTSTREAM_CONTRACTID =
                              "@mozilla.org/network/file-output-stream;1";
const NS_RDFSERVICE_CONTRACTID = "@mozilla.org/rdf/rdf-service;1";

// File read/write operations
const NS_RDONLY      = 0x01;
const NS_WRONLY      = 0x02;
const NS_CREATE_FILE = 0x08;
const NS_TRUNCATE    = 0x20;

// user -- read, write; group -- read; other -- read
const DEFAULT_FILE_PERMS = 00400 | 00200 | 00040 | 00004;

var gList   = null;
var gDS     = null;
var gSeq    = null;

var gNewServerRV = null;

var gNewServerButton  = null;
var gEditServerButton     = null;
var gRemoveServerButton   = null;

var LOCALSERVER_URL = "urn:localserver";
var gLocalServerResource = gRDF.GetResource(LOCALSERVER_URL);

/**
 * Initializes this preferences window.  
 * Called by XUL upon loading.
 */
function Startup()
{
    // set up the elements
    gList = document.getElementById("serverList");
    gNewServerButton  = document.getElementById("newServerButton");
    gEditServerButton     = document.getElementById("editServerButton");
    gRemoveServerButton   = document.getElementById("removeServerButton");
		gAddLocalServerButton = document.getElementById("addLocalServerButton");
		gRemoveLocalServerButton = document.getElementById("removeLocalServerButton");

		// get the preferences directory
    var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);  
    var file = fileLocator.get("ProfD", Components.interfaces.nsIFile);
  
		// append servers filename
    file.append(SERVERS_FILENAME);	

		// if the file does not exist, create it
		if (!file.exists()) {
			createNewAnnozillaServersFile(file);
		}		
		
		// get the data source
		var urlSpec = getURLSpecFromFile(file);
		gDS = gRDF.GetDataSourceBlocking(urlSpec);
		
		// intialize the listbox
		gList.database.AddDataSource(gDS);
		gList.setAttribute("ref", SERVERS_ROOT_URN);

		// build a seq container
    gSeq = RDFU.makeSeq(gDS, gRDF.GetResource(SERVERS_ROOT_URN));

		// setup localServer
		maintainLocalServer();
}

function maintainLocalServer() {
	var inList = isLocalServerInList();
	document.getElementById("addLocalServerButton").hidden = inList;
	document.getElementById("removeLocalServerButton").hidden = !inList;
}

function isLocalServerInList() {
	return gSeq.IndexOf(gLocalServerResource) != -1;
}

function addLocalServer() {

	gSeq.AppendElement(gLocalServerResource);

	// assert the literals for the localServer
	assertForResource(gLocalServerResource, "user", "noUser");
	assertForResource(gLocalServerResource, "password", "noPass");
	
	// flush the ds to disk.   
	var remoteDS = gDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
	remoteDS.Flush();
  
	maintainLocalServer();
}

function removeLocalServer() {
	deleteServerFromRDF(LOCALSERVER_URL);
	
	// rebuild the list
	gList.builder.rebuild();
	gList.setAttribute("ref", SERVERS_ROOT_URN);
	
	// setup localServer
	maintainLocalServer();
}

/**
 * Prepend the server namespace string to a property.
 *
 * @param aProperty String property
 * @return concatenated String
 */
function SERVER_RDF(aProperty)
{
    return "urn:annotations:serverNS#" + aProperty;
}

/**
 * Assert a literal for a resource in the global
 * datasource.
 *
 * @param aResource Resource to assert the literal for
 * @param aPropertyString literal to assert
 * @param aValueString value for that literal
 */
function assertForResource(aResource, aPropertyString, aValueString)
{
  var valueProperty = gRDF.GetResource(SERVER_RDF(aPropertyString));
  var serverLiteral = gRDF.GetLiteral(aValueString);
  gDS.Assert(aResource, valueProperty, serverLiteral, true);
}


/**
 * Create a new server entry.  Blocks upon
 * opening dialog window.
 */
function newServer()
{
	window.openDialog("chrome://annozilla/content/pref-annozilla-servers-new.xul", "newServer", "chrome,modal=yes,resizable=no");
	
	if (gNewServerRV) {
		gList.builder.rebuild();
		gList.setAttribute("ref", SERVERS_ROOT_URN);
		gNewServerRV = null;

		// setup localServer
		maintainLocalServer();
  }
	else {
		dump("No gNewServerRV!\n");
	}
}

/**
 * Remove the currently selected server.
 */
function removeServer()
{	
	if (gList.selectedItems && gList.selectedItems[0]) {
		var uri = gList.selectedItems[0].id;
		deleteServerFromRDF(uri);
		gList.setAttribute("ref", SERVERS_ROOT_URN);

		// setup localServer
		maintainLocalServer();
	}
}

/**
 * Edit the currently selected server.  Blocks upon opening
 * dialog window.
 */
function editServer()
{
	if (gList.selectedItems && gList.selectedItems[0]) {
		var uri = gList.selectedItems[0].id;
		if (uri == LOCALSERVER_URL) {
			window.alert("The LocalServer has no editable properties!");
		}
		else {
			window.openDialog("chrome://annozilla/content/pref-annozilla-servers-edit.xul", "serverEdit", "chrome,modal=yes,resizable=no", uri);
		}
	}
}

/**
 * Removes a server entry from the servers
 * RDF file.
 *
 * @param aResourceString String name of resource to remove
 */
function deleteServerFromRDF(aResourceString) {

	// find and remove the entry
	var element = gRDF.GetResource(aResourceString);
	if (gSeq.IndexOf(element) != -1) {
        try {
          gSeq.RemoveElement(element, true);
        }
        catch(e) {
          dump("Error removing element: " + e + "\n");
        } 
	}
	else {
		dump("Error: Could not find element in container.\n");
	}

  // remove the resource from the graph  

	// urns:  [ URN predicate function for namespace, [ "literal0", "literal1", ... ] ]
	var urns = [ [SERVER_RDF, ["user", "password"]] ];
	
	// iterate over all namespaces
	for (var i = 0; i < urns.length; i++) {

		var namespaceFunc = urns[i][0];
		var properties = urns[i][1];

		// iterate over all properties in the namespace
		for (var j = 0; j < properties.length; j++) {
			var propertyRes = gRDF.GetResource(namespaceFunc(properties[j]), true);
			var value = gDS.GetTarget(element, propertyRes, true);
			if (element && propertyRes && value) {
				gDS.Unassert(element, propertyRes, value, true);
			}
		}
	}
	
	// flush data source
  try {
    gDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
  } catch(e) {
		dump("Couldn't flush data-source.\n");
  }
}


/**
 * Prepend the server namespace to a property.
 *
 * @param aProperty String property
 * @return concatenated String
 */
function SERVER_RDF(aProperty)
{
    return "urn:annotations:serverNS#" + aProperty;
}


/**
 * Creates a new blank servers file.
 *
 * @param file nsIFile with location for new file
 * @return the nsIFile (evals to a nsILocalFile) created
 */
function createNewAnnozillaServersFile(file) {

	try {

		// initialize local file
    var localFile = Components.classes[NS_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
		
		localFile.initWithPath(file.path);

		if (localFile.exists() && 
				(localFile.isDirectory() || !localFile.isWritable())) {
			throw Components.results.NS_ERROR_FAILURE;		
		}
	
		// write flags
		var flags = NS_WRONLY | NS_CREATE_FILE | NS_TRUNCATE;

		// get a file output stream
		var fileStream = Components.classes[NS_LOCALFILEOUTPUTSTREAM_CONTRACTID].createInstance(Components.interfaces.nsIFileOutputStream);
    fileStream.init(localFile, flags, DEFAULT_FILE_PERMS, 0);

		// template for servers rdf
		var outputStr = "<?xml version=\"1.0\"?>\n" +
			"<RDF:RDF xmlns:s=\"urn:annotations:serverNS#\"  \n" +
			"   xmlns:NC=\"http://home.netscape.com/NC-rdf#\"  \n" +
			"   xmlns:RDF=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\n" +
			"   <RDF:Bag about=\"urn:annotations:servers\">\n" +
			"   </RDF:Bag>\n" +
			"</RDF:RDF>\n";

		fileStream.write(outputStr, outputStr.length);
		fileStream.close();

		return localFile;
	}
	catch (e) {
		dump("Error writing out new file: " + e + "\n");
		return undefined;
	}
}
	

function getURLSpecFromFile(file) {
    
    if (Components.classes["@mozilla.org/network/io-service;1"] && Components.interfaces.nsIIOService) {
	
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);

	// new version
	if (fileHandler.getURLSpecFromFile) {
	    return fileHandler.getURLSpecFromFile(file);
	}
	// old version in ioService
	else {
	    return ioService.getURLSpecFromFile(file);
	}
    }
    // really old version
    else {
	var file_url = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIFileURL);
	file_url.file = file;
	return file_url.spec;
    }
}
