/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Linky
 *
 * The Initial Developer of the Original Code is Henrik Gemal.
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Henrik Gemal <linky@gemal.dk> http://gemal.dk
 *   Doug Daniels <rainking@alumni.rice.edu>
 *   Matthew Wilson <matthew@mjwilson.demon.co.uk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// product related settings
// you can change these
const myProductName = "Annozilla";
const myProductRegKey = "annozilla";
const myProductRegVersion = "0.5beta";
const myProductRegVersionBuild = "20040412";
const myJarFileName = "annozilla.jar";
const myJarFileSize = 96; // the filesize in KB of myJarFileName
const myJSFileName = ""; // the default prefs file. set it to "" if not used
const myJSFileSize = 0; // the filesize in KB of myJSFileName
const myLocales = new Array("en-US"); // the locales available
const myJarCheckDupe = 1; // check if myJarFileName exist both in personal chrome folder and global chrome folder

// global settings
// dont change these
const sysChromeGlobal = getFolder("Chrome");
const sysChromeGlobalJar = getFolder("Chrome", myJarFileName);
const sysChromeUser = getFolder("Current User","chrome");
const sysChromeUserJar = getFolder("Current User", "chrome/" + myJarFileName);
const errPage = "http://devedge.netscape.com/library/manuals/2001/xpinstall/1.0/err.html";

// verify disk space in kilobytes
function verifyDiskSpace(dirPath, spaceRequired) {
	var spaceAvailable = fileGetDiskSpaceAvailable(dirPath);
	if (parseInt(spaceAvailable/1024) < spaceRequired) {
		logComment("Insufficient disk space: " + dirPath + "\n\trequired : " + spaceRequired + " K\n\tavailable: " + spaceAvailable + " K");
		return(false);
	} else {
		return(true);
	}
}

var inst;
// inst is the installation method:
//		0 means that the installation was cancelled.
//		1 is in the global chrome folder. This *has* to be the default!
//		2 in in the personal profile folder. This is only supported for newer builds.

if (myJarCheckDupe && File.exists(sysChromeGlobalJar) && File.exists(sysChromeUserJar)) {
	alert(myProductName + " is installed in BOTH the personal chrome folder:\n" + sysChromeUserJar + "\nand the global chrome folder:\n" + sysChromeGlobalJar + "\n\nThis will cause weird behaviour. Please delete one of the instances!");
	inst = 0;
} else {
	// Can we install in the personal profile folder?
	if (buildID && buildID >= 2003030700) {

		var ans;
		// Check if extension already is installed
		if (File.exists(sysChromeGlobalJar)) {
                        // Automatically install into the global area without asking
			inst = 1;
		} else if (File.exists(sysChromeUserJar)) {
                        // Automatically install into the user area without asking
			inst = 2;
		}

		if (!inst) {
                    var ans = confirm ("You can install " + myProductName + " " + myProductRegVersion + " in two ways:\n\n1) in the global chrome folder. This will make " + myProductName + " available in all profiles (i.e. to all users).\n\n2) in the personal profile folder. This will make " + myProductName + " available only in the current profile. (i.e. to the current user)\n\nPress OK to continue or Cancel to abandon the installation");
                    if (ans) {
                        ans = confirm("Select OK to install in the global chrome folder:\n" + sysChromeGlobal + "\n(This is the recommended install option.)\nOr select Cancel to install in your personal profile folder:\n" + sysChromeUser);
                        if (ans) {
                            inst = 1;
                        } else {
                            inst = 2;
                        }
                    } else {
                        // Installation abandoned
                        inst = 0;
                    }
                }
	} else {
                // Build ID says we don't know how to install in user area,
                // so automatically install in global area without asking
                inst = 1;
	}
}

// did the user cancel?
if (inst) {
	var err;
	var err_tmp;
	// init the installation
	err_tmp = initInstall(myProductName, myProductRegKey, myProductRegVersion + "." + myProductRegVersionBuild);
	if (err_tmp) {
		err = err_tmp;
	}

	logComment(myProductName + " version " + myProductRegVersion + " being installed on " + buildID);
	logComment("Installation method is " + inst);

	// check if there's disk space for myJSFileSize
	if (!err && !verifyDiskSpace(sysChromeGlobal,myJSFileSize)) {
		err = INSUFFICIENT_DISK_SPACE;
	}

	// check if there's disk space for myJarFileSize
	if (!err && !verifyDiskSpace((inst == 1 ? sysChromeGlobal : sysChromeUser), myJarFileSize)) {
		err = INSUFFICIENT_DISK_SPACE;
	}

	// add the default prefs, but only if we have to install into global chrome
	if (!err && inst == 1 && myJSFileName) {
		err_tmp = addFile(myProductName, myJSFileName, getFolder("Program","defaults/pref"), myJSFileName);
		if (err_tmp) {
			logComment("Problem adding the pref defaults to: " + getFolder("Program", "defaults/pref") + "\n\tError code: " + err_tmp);
			err = err_tmp;
		}
	}

	// add the jar file
	if (!err) {
		if (inst == 2) {
			err_tmp = addFile(myProductName, myJarFileName, sysChromeUser, "");
		} else {
			err_tmp = addFile(myProductName, myJarFileName, sysChromeGlobal, "");
		}
		if (err_tmp) {
			logComment("Problem adding jar file. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			logComment("ok adding jar file.");
		}
	}

	// register the content
	if (!err) {
		if (inst == 2) {
			err_tmp = registerChrome(PACKAGE | PROFILE_CHROME, sysChromeUserJar, "content/" + myProductRegKey + "/");
		} else {
			err_tmp = registerChrome(PACKAGE | DELAYED_CHROME, sysChromeGlobalJar, "content/" + myProductRegKey + "/");
		}
		if (err_tmp) {
			logComment("Problem registering the content chrome. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			logComment("ok registering the content chrome.");
		}
	}

	// register the locale
	if (!err) {
		if (inst == 2) {
			for (var i = 0; i < myLocales.length; i++) {
				err_tmp = registerChrome(LOCALE | PROFILE_CHROME, sysChromeUserJar, "locale/" + myLocales[i] + "/" + myProductRegKey + "/");
				if (err_tmp)
					break;
			}
		} else {
			for (var i = 0; i < myLocales.length; i++) {
				err_tmp = registerChrome(LOCALE | DELAYED_CHROME, sysChromeGlobalJar, "locale/" + myLocales[i] + "/" + myProductRegKey + "/");
				if (err_tmp) {
					break;
				}
			}
		}
		if (err_tmp) {
			logComment("Problem registering the locale chrome. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			logComment("ok registering the locale chrome.");
		}
	}

	// do the install
	if (!err) {
		err_tmp = performInstall();
		if (err_tmp < 0) {
			logComment("Problem performing install. Error code: " + err_tmp);
			err = err_tmp;
		} else {
			alert(myProductName + " version " + myProductRegVersion + " has been successfully installed!\nYou must restart your browser to complete installation.");
		}
	}

	// did it work?
	if (err) {
		logComment("Problem installing. Error code: " + err + ". Error codes can been seen at: " + errPage);
		alert(myProductName + " version " + myProductRegVersion + " was not installed!\nError code: " + err + "\n\nError codes can been seen at:\n" + errPage)
		cancelInstall(err);
	}
} else {
	cancelInstall();
}
